@@ -16,7 +16,7 @@ mod function_wrapper_cpp;
 pub(crate) mod type_to_cpp;
 
 use crate::{
-    conversion::analysis::fun::{function_wrapper::CppFunctionKind, FnAnalysisBody},
+    conversion::analysis::fun::{function_wrapper::CppFunctionKind, FnAnalysis},
     types::{make_ident, QualifiedName},
     CppFilePair,
 };
@@ -28,7 +28,7 @@ use type_to_cpp::{original_name_map_from_apis, type_to_cpp, CppNameMap};
 use super::{
     analysis::fun::{
         function_wrapper::{CppFunction, CppFunctionBody},
-        FnAnalysis,
+        FnPhase,
     },
     api::{Api, SubclassName},
     ConvertError,
@@ -59,6 +59,10 @@ impl Header {
             format!("#include \"{}\"", self.name)
         }
     }
+
+    fn is_system(&self) -> bool {
+        self.system
+    }
 }
 
 enum ConversionDirection {
@@ -86,15 +90,22 @@ pub(crate) struct CppCodeGenerator<'a> {
     inclusions: String,
     original_name_map: CppNameMap,
     config: &'a IncludeCppConfig,
+    suppress_system_headers: bool,
 }
 
 impl<'a> CppCodeGenerator<'a> {
     pub(crate) fn generate_cpp_code(
         inclusions: String,
-        apis: &[Api<FnAnalysis>],
+        apis: &[Api<FnPhase>],
         config: &'a IncludeCppConfig,
+        suppress_system_headers: bool,
     ) -> Result<Option<CppFilePair>, ConvertError> {
-        let mut gen = CppCodeGenerator::new(inclusions, original_name_map_from_apis(apis), config);
+        let mut gen = CppCodeGenerator::new(
+            inclusions,
+            original_name_map_from_apis(apis),
+            config,
+            suppress_system_headers,
+        );
         // The 'filter' on the following line is designed to ensure we don't accidentally
         // end up out of sync with needs_cpp_codegen
         gen.add_needs(apis.iter().filter(|api| api.needs_cpp_codegen()))?;
@@ -105,19 +116,21 @@ impl<'a> CppCodeGenerator<'a> {
         inclusions: String,
         original_name_map: CppNameMap,
         config: &'a IncludeCppConfig,
+        suppress_system_headers: bool,
     ) -> Self {
         CppCodeGenerator {
             additional_functions: Vec::new(),
             inclusions,
             original_name_map,
             config,
+            suppress_system_headers,
         }
     }
 
     // It's important to keep this in sync with Api::needs_cpp_codegen.
     fn add_needs<'b>(
         &mut self,
-        apis: impl Iterator<Item = &'a Api<FnAnalysis>>,
+        apis: impl Iterator<Item = &'a Api<FnPhase>>,
     ) -> Result<(), ConvertError> {
         let mut constructors_by_subclass: HashMap<SubclassName, Vec<&CppFunction>> = HashMap::new();
         let mut methods_by_subclass: HashMap<SubclassName, Vec<&CppFunction>> = HashMap::new();
@@ -127,7 +140,7 @@ impl<'a> CppCodeGenerator<'a> {
                 Api::StringConstructor { .. } => self.generate_string_constructor(),
                 Api::Function {
                     analysis:
-                        FnAnalysisBody {
+                        FnAnalysis {
                             cpp_wrapper: Some(cpp_wrapper),
                             ..
                         },
@@ -177,42 +190,53 @@ impl<'a> CppCodeGenerator<'a> {
         if self.additional_functions.is_empty() {
             None
         } else {
-            let headers: HashSet<Header> = self
-                .additional_functions
-                .iter()
-                .map(|x| x.headers.iter().cloned())
-                .flatten()
-                .collect();
-            let headers = headers.iter().map(|x| x.include_stmt()).join("\n");
-            let cpp_headers: HashSet<Header> = self
-                .additional_functions
-                .iter()
-                .map(|x| x.cpp_headers.iter().cloned())
-                .flatten()
-                .collect();
-            let cpp_headers = cpp_headers.iter().map(|x| x.include_stmt()).join("\n");
+            let headers = self.collect_headers(|additional_need| &additional_need.headers);
+            let cpp_headers = self.collect_headers(|additional_need| &additional_need.cpp_headers);
             let type_definitions = self.concat_additional_items(|x| x.type_definition.as_ref());
             let declarations = self.concat_additional_items(|x| x.declaration.as_ref());
             let declarations = format!(
                 "#ifndef __AUTOCXXGEN_H__\n#define __AUTOCXXGEN_H__\n\n{}\n{}\n{}\n{}#endif // __AUTOCXXGEN_H__\n",
                 headers, self.inclusions, type_definitions, declarations
             );
-            let definitions = self.concat_additional_items(|x| x.definition.as_ref());
             log::info!("Additional C++ decls:\n{}", declarations);
             let header_name = format!("autocxxgen_{}.h", self.config.get_mod_name());
-            let definitions = format!(
-                "#include \"{}\"\n{}\n{}",
-                header_name, cpp_headers, definitions
-            );
-            log::info!("Additional C++ defs:\n{}", definitions);
+            let implementation = if self
+                .additional_functions
+                .iter()
+                .any(|x| x.definition.is_some())
+            {
+                let definitions = self.concat_additional_items(|x| x.definition.as_ref());
+                let definitions = format!(
+                    "#include \"{}\"\n{}\n{}",
+                    header_name, cpp_headers, definitions
+                );
+                log::info!("Additional C++ defs:\n{}", definitions);
+                Some(definitions.into_bytes())
+            } else {
+                None
+            };
             Some(CppFilePair {
                 header: declarations.into_bytes(),
-                implementation: definitions.into_bytes(),
+                implementation,
                 header_name,
             })
         }
     }
 
+    fn collect_headers<F>(&self, filter: F) -> String
+    where
+        F: Fn(&AdditionalFunction) -> &[Header],
+    {
+        let cpp_headers: HashSet<_> = self
+            .additional_functions
+            .iter()
+            .map(|x| filter(x).iter())
+            .flatten()
+            .filter(|x| !self.suppress_system_headers || !x.is_system())
+            .collect(); // uniqify
+        cpp_headers.iter().map(|x| x.include_stmt()).join("\n")
+    }
+
     fn concat_additional_items<F>(&self, field_access: F) -> String
     where
         F: FnMut(&AdditionalFunction) -> Option<&String>,
@@ -329,7 +353,7 @@ impl<'a> CppCodeGenerator<'a> {
         };
         let declaration = format!("{} {}({}){}", ret_type, name, args, constness);
         let qualification = if let Some(qualification) = &details.qualification {
-            format!("{}::", qualification.to_string())
+            format!("{}::", qualification.to_cpp_name())
         } else {
             "".to_string()
         };
@@ -343,16 +367,21 @@ impl<'a> CppCodeGenerator<'a> {
             .enumerate()
             .map(|(counter, conv)| match conversion_direction {
                 ConversionDirection::RustCallsCpp => {
-                    conv.cpp_conversion(&get_arg_name(counter), &self.original_name_map)
+                    conv.cpp_conversion(&get_arg_name(counter), &self.original_name_map, false)
                 }
                 ConversionDirection::CppCallsCpp => Ok(get_arg_name(counter)),
-                ConversionDirection::CppCallsRust => conv
-                    .inverse()
-                    .cpp_conversion(&get_arg_name(counter), &self.original_name_map),
+                ConversionDirection::CppCallsRust => conv.inverse().cpp_conversion(
+                    &get_arg_name(counter),
+                    &self.original_name_map,
+                    false,
+                ),
             })
             .collect();
         let mut arg_list = arg_list?.into_iter();
         let receiver = if is_a_method { arg_list.next() } else { None };
+        if matches!(&details.payload, CppFunctionBody::ConstructSuperclass(_)) {
+            arg_list.next();
+        }
         let arg_list = if details.pass_obs_field {
             std::iter::once("*obs".to_string())
                 .chain(arg_list)
@@ -360,17 +389,23 @@ impl<'a> CppCodeGenerator<'a> {
         } else {
             arg_list.join(", ")
         };
-        let mut underlying_function_call = match &details.payload {
-            CppFunctionBody::Constructor => arg_list,
+        let (mut underlying_function_call, field_assignments) = match &details.payload {
+            CppFunctionBody::Constructor => (arg_list, "".to_string()),
             CppFunctionBody::FunctionCall(ns, id) => match receiver {
-                Some(receiver) => format!("{}.{}({})", receiver, id.to_string(), arg_list),
+                Some(receiver) => (
+                    format!("{}.{}({})", receiver, id.to_string(), arg_list),
+                    "".to_string(),
+                ),
                 None => {
                     let underlying_function_call = ns
                         .into_iter()
                         .cloned()
                         .chain(std::iter::once(id.to_string()))
                         .join("::");
-                    format!("{}({})", underlying_function_call, arg_list)
+                    (
+                        format!("{}({})", underlying_function_call, arg_list),
+                        "".to_string(),
+                    )
                 }
             },
             CppFunctionBody::StaticMethodCall(ns, ty_id, fn_id) => {
@@ -379,32 +414,45 @@ impl<'a> CppCodeGenerator<'a> {
                     .cloned()
                     .chain([ty_id.to_string(), fn_id.to_string()].iter().cloned())
                     .join("::");
-                format!("{}({})", underlying_function_call, arg_list)
+                (
+                    format!("{}({})", underlying_function_call, arg_list),
+                    "".to_string(),
+                )
             }
-            CppFunctionBody::AssignSubclassHolderField => "".to_string(),
+            CppFunctionBody::ConstructSuperclass(_) => ("".to_string(), arg_list),
         };
         if let Some(ret) = &details.return_conversion {
             underlying_function_call = format!(
                 "return {}",
                 match conversion_direction {
-                    ConversionDirection::RustCallsCpp =>
-                        ret.cpp_conversion(&underlying_function_call, &self.original_name_map)?,
+                    ConversionDirection::RustCallsCpp => ret.cpp_conversion(
+                        &underlying_function_call,
+                        &self.original_name_map,
+                        true
+                    )?,
                     ConversionDirection::CppCallsCpp => underlying_function_call,
-                    ConversionDirection::CppCallsRust => ret
-                        .inverse()
-                        .cpp_conversion(&underlying_function_call, &self.original_name_map)?,
+                    ConversionDirection::CppCallsRust => ret.inverse().cpp_conversion(
+                        &underlying_function_call,
+                        &self.original_name_map,
+                        true
+                    )?,
                 }
             );
         };
         if !underlying_function_call.is_empty() {
             underlying_function_call = format!("{};", underlying_function_call);
         }
-        let field_assignments = if let CppFunctionBody::AssignSubclassHolderField = &details.payload
-        {
-            ": obs(std::move(arg0))"
-        } else {
-            ""
-        };
+        let field_assignments =
+            if let CppFunctionBody::ConstructSuperclass(superclass_name) = &details.payload {
+                let superclass_assignments = if field_assignments.is_empty() {
+                    "".to_string()
+                } else {
+                    format!("{}({}), ", superclass_name, field_assignments)
+                };
+                format!(": {}obs(std::move(arg0))", superclass_assignments)
+            } else {
+                "".into()
+            };
         let definition_after_sig =
             format!("{} {{ {} }}", field_assignments, underlying_function_call,);
         let (declaration, definition) = if requires_rust_declarations {
@@ -476,8 +524,11 @@ impl<'a> CppCodeGenerator<'a> {
             let id = make_ident(method.wrapper_function_name.to_string());
             let mut super_method = method.clone();
             super_method.pass_obs_field = false;
-            super_method.wrapper_function_name =
-                SubclassName::get_super_fn_name(&method.wrapper_function_name.to_string());
+            super_method.wrapper_function_name = SubclassName::get_super_fn_name(
+                superclass.get_namespace(),
+                &method.wrapper_function_name.to_string(),
+            )
+            .get_final_ident();
             super_method.payload = CppFunctionBody::StaticMethodCall(
                 superclass.get_namespace().clone(),
                 superclass.get_final_ident(),
@@ -508,7 +559,7 @@ impl<'a> CppCodeGenerator<'a> {
             let mut fn_impl = self.generate_cpp_function_inner(
                 constructor,
                 false,
-                ConversionDirection::RustCallsCpp,
+                ConversionDirection::CppCallsCpp,
                 false,
             )?;
             let decl = fn_impl.declaration.take().unwrap();
