@@ -23,23 +23,25 @@ use syn::{
 };
 
 use super::{
-    doc_attr::get_doc_attr,
     unqualify::{unqualify_params, unqualify_ret_type},
     RsCodegenResult, Use,
 };
-use crate::{conversion::api::FuncToConvert, types::make_ident};
 use crate::{
     conversion::{
-        analysis::fun::{ArgumentAnalysis, FnAnalysisBody, FnKind, MethodKind, RustRenameStrategy},
+        analysis::fun::{ArgumentAnalysis, FnAnalysis, FnKind, MethodKind, RustRenameStrategy},
         api::ImplBlockDetails,
     },
     types::{Namespace, QualifiedName},
 };
+use crate::{
+    conversion::{api::FuncToConvert, codegen_rs::lifetime::add_explicit_lifetime_if_necessary},
+    types::make_ident,
+};
 
 pub(super) fn gen_function(
     ns: &Namespace,
     fun: FuncToConvert,
-    analysis: FnAnalysisBody,
+    analysis: FnAnalysis,
     cpp_call_name: String,
 ) -> RsCodegenResult {
     let cxxbridge_name = analysis.cxxbridge_name;
@@ -50,7 +52,7 @@ pub(super) fn gen_function(
     let params = analysis.params;
     let vis = analysis.vis;
     let kind = analysis.kind;
-    let doc_attr = get_doc_attr(&fun.item.attrs);
+    let doc_attr = fun.doc_attr;
 
     let mut cpp_name_attr = Vec::new();
     let mut impl_entry = None;
@@ -68,10 +70,12 @@ pub(super) fn gen_function(
         _ => Vec::new(),
     };
     let mut materialization = match kind {
-        FnKind::Method(..) => Use::Unused,
+        FnKind::Method(..) => None,
         FnKind::Function => match analysis.rust_rename_strategy {
-            RustRenameStrategy::RenameInOutputMod(alias) => Use::UsedFromCxxBridgeWithAlias(alias),
-            _ => Use::UsedFromCxxBridge,
+            RustRenameStrategy::RenameInOutputMod(alias) => {
+                Some(Use::UsedFromCxxBridgeWithAlias(alias))
+            }
+            _ => Some(Use::UsedFromCxxBridge),
         },
     };
     let any_param_needs_rust_conversion = param_details
@@ -94,13 +98,13 @@ pub(super) fn gen_function(
             ));
         } else {
             // Generate plain old function
-            materialization = Use::Custom(generate_function_impl(
+            materialization = Some(Use::Custom(generate_function_impl(
                 &param_details,
                 &rust_name,
                 &ret_type,
                 &unsafety,
                 &doc_attr,
-            ));
+            )));
         }
     }
     if cxxbridge_name != cpp_call_name && !wrapper_function_needed {
@@ -110,6 +114,10 @@ pub(super) fn gen_function(
             ))
             .unwrap();
     }
+    // In very rare occasions, we might need to give an explicit lifetime.
+    let (lifetime_tokens, params, ret_type) =
+        add_explicit_lifetime_if_necessary(&param_details, params, &ret_type);
+
     // Finally - namespace support. All the Types in everything
     // above this point are fully qualified. We need to unqualify them.
     // We need to do that _after_ the above wrapper_function_needed
@@ -119,7 +127,7 @@ pub(super) fn gen_function(
     // and the following code will act to unqualify only those types
     // which the user has declared.
     let params = unqualify_params(params);
-    let ret_type = unqualify_ret_type(ret_type);
+    let ret_type = unqualify_ret_type(ret_type.into_owned());
     // And we need to make an attribute for the namespace that the function
     // itself is in.
     let namespace_attr = if ns.is_empty() || wrapper_function_needed {
@@ -138,7 +146,7 @@ pub(super) fn gen_function(
         #(#rust_name_attr)*
         #(#cpp_name_attr)*
         #doc_attr
-        #vis #unsafety fn #cxxbridge_name ( #params ) #ret_type;
+        #vis #unsafety fn #cxxbridge_name #lifetime_tokens ( #params ) #ret_type;
     ));
     RsCodegenResult {
         extern_c_mod_items: vec![extern_c_mod_item],
@@ -146,7 +154,7 @@ pub(super) fn gen_function(
         global_items: Vec::new(),
         bindgen_mod_items: Vec::new(),
         impl_entry,
-        materialization,
+        materializations: materialization.into_iter().collect(),
         extern_rust_mod_items: Vec::new(),
     }
 }
@@ -186,11 +194,13 @@ fn generate_method_impl(
     doc_attr: &Option<Attribute>,
 ) -> Box<ImplBlockDetails> {
     let (wrapper_params, arg_list) = generate_arg_lists(param_details, is_constructor);
+    let (lifetime_tokens, wrapper_params, ret_type) =
+        add_explicit_lifetime_if_necessary(param_details, wrapper_params, ret_type);
     let rust_name = make_ident(&rust_name);
     Box::new(ImplBlockDetails {
         item: ImplItem::Method(parse_quote! {
             #doc_attr
-            pub #unsafety fn #rust_name ( #wrapper_params ) #ret_type {
+            pub #unsafety fn #rust_name #lifetime_tokens ( #wrapper_params ) #ret_type {
                 cxxbridge::#cxxbridge_name ( #(#arg_list),* )
             }
         }),
