@@ -12,31 +12,32 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-mod doc_attr;
 mod fun_codegen;
 mod function_wrapper_rs;
 mod impl_item_creator;
+mod lifetime;
 mod namespace_organizer;
 mod non_pod_struct;
 pub(crate) mod unqualify;
 
-use std::collections::HashMap;
+use std::collections::{HashMap, HashSet};
 
 use autocxx_parser::IncludeCppConfig;
-// The following should not need to be exposed outside
-// codegen_rs but currently Rust codegen happens everywhere... TODO
-pub(crate) use non_pod_struct::make_non_pod;
 
 use proc_macro2::{Span, TokenStream};
 use syn::{
-    parse_quote, punctuated::Punctuated, token::Comma, Expr, FnArg, ForeignItem, ForeignItemFn,
-    Ident, ImplItem, Item, ItemForeignMod, ItemMod, Pat, ReturnType, TraitItem,
+    parse_quote, punctuated::Punctuated, token::Comma, Attribute, Expr, FnArg, ForeignItem,
+    ForeignItemFn, Ident, ImplItem, Item, ItemForeignMod, ItemMod, Pat, ReturnType, TraitItem,
 };
 
 use crate::{
     conversion::{
         analysis::fun::MethodKind,
-        codegen_rs::unqualify::{unqualify_params, unqualify_ret_type},
+        codegen_rs::{
+            non_pod_struct::{make_non_pod, new_non_pod_struct},
+            unqualify::{unqualify_params, unqualify_ret_type},
+        },
+        doc_attr::get_doc_attr,
     },
     known_types::known_types,
     types::{make_ident, Namespace, QualifiedName},
@@ -46,21 +47,21 @@ use impl_item_creator::create_impl_items;
 use self::{
     fun_codegen::gen_function,
     namespace_organizer::{HasNs, NamespaceEntries},
-    non_pod_struct::new_non_pod_struct,
 };
 
 use super::{
-    analysis::fun::{FnAnalysis, ReceiverMutability},
-    api::{AnalysisPhase, Api, ImplBlockDetails, SubclassName, TypeKind, TypedefKind},
-};
-use super::{
-    analysis::fun::{FnAnalysisBody, FnKind},
+    analysis::fun::{FnAnalysis, FnKind},
+    api::RustSubclassFnDetails,
     codegen_cpp::type_to_cpp::{
         namespaced_name_using_original_name_map, original_name_map_from_apis, CppNameMap,
     },
 };
+use super::{
+    analysis::fun::{FnPhase, ReceiverMutability},
+    api::{AnalysisPhase, Api, ImplBlockDetails, SubclassName, TypeKind, TypedefKind},
+};
 use super::{convert_error::ErrorContext, ConvertError};
-use quote::quote;
+use quote::{quote, ToTokens};
 
 unzip_n::unzip_n!(pub 4);
 
@@ -68,24 +69,18 @@ unzip_n::unzip_n!(pub 4);
 /// for actual end-user use.
 #[derive(Clone)]
 enum Use {
-    /// Not used
-    Unused,
     /// Uses from cxx::bridge
     UsedFromCxxBridge,
     /// 'use' points to cxx::bridge with a different name
     UsedFromCxxBridgeWithAlias(Ident),
     /// 'use' directive points to bindgen
     UsedFromBindgen,
+    /// 'use' a specific name from bindgen.
+    SpecificNameFromBindgen(Ident),
     /// Some kind of custom item
     Custom(Box<Item>),
 }
 
-impl Default for Use {
-    fn default() -> Self {
-        Self::Unused
-    }
-}
-
 fn get_string_items() -> Vec<Item> {
     [
         Item::Trait(parse_quote! {
@@ -128,13 +123,13 @@ fn get_string_items() -> Vec<Item> {
     .to_vec()
 }
 
-#[derive(Debug)]
 struct SuperclassMethod {
     name: Ident,
     params: Punctuated<FnArg, Comma>,
     param_names: Vec<Pat>,
     ret_type: ReturnType,
     receiver_mutability: ReceiverMutability,
+    requires_unsafe: bool,
 }
 
 /// Type which handles generation of Rust code.
@@ -150,7 +145,7 @@ pub(crate) struct RsCodeGenerator<'a> {
 impl<'a> RsCodeGenerator<'a> {
     /// Generate code for a set of APIs that was discovered during parsing.
     pub(crate) fn generate_rs_code(
-        all_apis: Vec<Api<FnAnalysis>>,
+        all_apis: Vec<Api<FnPhase>>,
         include_list: &'a [String],
         bindgen_mod: ItemMod,
         config: &'a IncludeCppConfig,
@@ -164,18 +159,24 @@ impl<'a> RsCodeGenerator<'a> {
         c.rs_codegen(all_apis)
     }
 
-    fn rs_codegen(mut self, all_apis: Vec<Api<FnAnalysis>>) -> Vec<Item> {
+    fn rs_codegen(mut self, all_apis: Vec<Api<FnPhase>>) -> Vec<Item> {
         // ... and now let's start to generate the output code.
         // First off, when we generate structs we may need to add some methods
         // if they're superclasses.
-        let mut methods_by_superclass = self.accumulate_superclass_methods(&all_apis);
+        let methods_by_superclass = self.accumulate_superclass_methods(&all_apis);
+        let subclasses_with_a_single_trivial_constructor =
+            find_trivially_constructed_subclasses(&all_apis);
         // Now let's generate the Rust code.
         let (rs_codegen_results_and_namespaces, additional_cpp_needs): (Vec<_>, Vec<_>) = all_apis
             .into_iter()
             .map(|api| {
                 let more_cpp_needed = api.needs_cpp_codegen();
                 let name = api.name().clone();
-                let gen = self.generate_rs_for_api(api, &mut methods_by_superclass);
+                let gen = self.generate_rs_for_api(
+                    api,
+                    &methods_by_superclass,
+                    &subclasses_with_a_single_trivial_constructor,
+                );
                 ((name, gen), more_cpp_needed)
             })
             .unzip();
@@ -257,19 +258,19 @@ impl<'a> RsCodeGenerator<'a> {
 
     fn accumulate_superclass_methods(
         &self,
-        apis: &[Api<FnAnalysis>],
-    ) -> HashMap<String, Vec<SuperclassMethod>> {
+        apis: &[Api<FnPhase>],
+    ) -> HashMap<QualifiedName, Vec<SuperclassMethod>> {
         let mut results = HashMap::new();
         results.extend(
             self.config
                 .superclasses()
-                .map(|sc| (sc.clone(), Vec::new())),
+                .map(|sc| (QualifiedName::new_from_cpp_name(sc), Vec::new())),
         );
         for api in apis {
             if let Api::Function {
                 name,
                 analysis:
-                    FnAnalysisBody {
+                    FnAnalysis {
                         kind: FnKind::Method(receiver, method_kind),
                         params,
                         ret_type,
@@ -282,7 +283,7 @@ impl<'a> RsCodeGenerator<'a> {
                 match method_kind {
                     MethodKind::Virtual(receiver_mutability)
                     | MethodKind::PureVirtual(receiver_mutability) => {
-                        let list = results.get_mut(&receiver.to_cpp_name());
+                        let list = results.get_mut(receiver);
                         if let Some(list) = list {
                             let param_names =
                                 param_details.iter().map(|pd| pd.name.clone()).collect();
@@ -292,6 +293,7 @@ impl<'a> RsCodeGenerator<'a> {
                                 ret_type: ret_type.clone(),
                                 param_names,
                                 receiver_mutability: receiver_mutability.clone(),
+                                requires_unsafe: param_details.iter().any(|pd| pd.requires_unsafe),
                             })
                         }
                     }
@@ -345,17 +347,20 @@ impl<'a> RsCodeGenerator<'a> {
         output_items: &mut Vec<Item>,
     ) {
         for (name, codegen) in ns_entries.entries() {
-            match &codegen.materialization {
-                Use::UsedFromCxxBridgeWithAlias(alias) => {
-                    output_items.push(Self::generate_cxx_use_stmt(name, Some(alias)))
+            output_items.extend(codegen.materializations.iter().map(|materialization| {
+                match materialization {
+                    Use::UsedFromCxxBridgeWithAlias(alias) => {
+                        Self::generate_cxx_use_stmt(name, Some(alias))
+                    }
+                    Use::UsedFromCxxBridge => Self::generate_cxx_use_stmt(name, None),
+                    Use::UsedFromBindgen => Self::generate_bindgen_use_stmt(name),
+                    Use::SpecificNameFromBindgen(id) => {
+                        let name = QualifiedName::new(name.get_namespace(), id.clone());
+                        Self::generate_bindgen_use_stmt(&name)
+                    }
+                    Use::Custom(item) => *item.clone(),
                 }
-                Use::UsedFromCxxBridge => {
-                    output_items.push(Self::generate_cxx_use_stmt(name, None))
-                }
-                Use::UsedFromBindgen => output_items.push(Self::generate_bindgen_use_stmt(name)),
-                Use::Unused => {}
-                Use::Custom(item) => output_items.push(*item.clone()),
-            };
+            }));
         }
         for (child_name, child_ns_entries) in ns_entries.children() {
             if child_ns_entries.is_empty() {
@@ -460,8 +465,9 @@ impl<'a> RsCodeGenerator<'a> {
 
     fn generate_rs_for_api(
         &self,
-        api: Api<FnAnalysis>,
-        associated_methods: &mut HashMap<String, Vec<SuperclassMethod>>,
+        api: Api<FnPhase>,
+        associated_methods: &HashMap<QualifiedName, Vec<SuperclassMethod>>,
+        subclasses_with_a_single_trivial_constructor: &HashSet<QualifiedName>,
     ) -> RsCodegenResult {
         let name = api.name().clone();
         let id = name.get_final_ident();
@@ -477,32 +483,12 @@ impl<'a> RsCodeGenerator<'a> {
                     global_items: get_string_items(),
                     bindgen_mod_items: Vec::new(),
                     impl_entry: None,
-                    materialization: Use::UsedFromCxxBridgeWithAlias(make_ident("make_string")),
+                    materializations: vec![Use::UsedFromCxxBridgeWithAlias(make_ident(
+                        "make_string",
+                    ))],
                     extern_rust_mod_items: Vec::new(),
                 }
             }
-            Api::ConcreteType { .. } => RsCodegenResult {
-                global_items: self.generate_extern_type_impl(TypeKind::NonPod, &name),
-                bridge_items: create_impl_items(&id, self.config),
-                extern_c_mod_items: vec![ForeignItem::Verbatim(
-                    self.generate_cxxbridge_type(&name),
-                )],
-                bindgen_mod_items: vec![Item::Struct(new_non_pod_struct(id.clone()))],
-                impl_entry: None,
-                materialization: Use::Unused,
-                extern_rust_mod_items: Vec::new(),
-            },
-            Api::ForwardDeclaration { .. } => RsCodegenResult {
-                extern_c_mod_items: vec![ForeignItem::Verbatim(
-                    self.generate_cxxbridge_type(&name),
-                )],
-                bridge_items: Vec::new(),
-                global_items: self.generate_extern_type_impl(TypeKind::NonPod, &name),
-                bindgen_mod_items: vec![Item::Struct(new_non_pod_struct(id))],
-                impl_entry: None,
-                materialization: Use::UsedFromCxxBridge,
-                extern_rust_mod_items: Vec::new(),
-            },
             Api::Function { fun, analysis, .. } => {
                 gen_function(name.get_namespace(), *fun, analysis, cpp_call_name)
             }
@@ -512,7 +498,7 @@ impl<'a> RsCodeGenerator<'a> {
                 bridge_items: Vec::new(),
                 extern_c_mod_items: Vec::new(),
                 bindgen_mod_items: vec![Item::Const(const_item)],
-                materialization: Use::UsedFromBindgen,
+                materializations: vec![Use::UsedFromBindgen],
                 extern_rust_mod_items: Vec::new(),
             },
             Api::Typedef { analysis, .. } => RsCodegenResult {
@@ -524,15 +510,31 @@ impl<'a> RsCodeGenerator<'a> {
                     TypedefKind::Use(use_item) => Item::Use(use_item),
                 }],
                 impl_entry: None,
-                materialization: Use::UsedFromBindgen,
+                materializations: vec![Use::UsedFromBindgen],
                 extern_rust_mod_items: Vec::new(),
             },
             Api::Struct { item, analysis, .. } => {
-                let methods = associated_methods.get(&id.to_string());
-                self.generate_type(&name, id, item, analysis.kind, Item::Struct, methods)
+                let doc_attr = get_doc_attr(&item.attrs);
+                self.generate_type(
+                    &name,
+                    id,
+                    analysis.kind,
+                    || Some((Item::Struct(item), doc_attr)),
+                    associated_methods,
+                )
             }
             Api::Enum { item, .. } => {
-                self.generate_type(&name, id, item, TypeKind::Pod, Item::Enum, None)
+                let doc_attr = get_doc_attr(&item.attrs);
+                self.generate_type(
+                    &name,
+                    id,
+                    TypeKind::Pod,
+                    || Some((Item::Enum(item), doc_attr)),
+                    associated_methods,
+                )
+            }
+            Api::ForwardDeclaration { .. } | Api::ConcreteType { .. } => {
+                self.generate_type(&name, id, TypeKind::Abstract, || None, associated_methods)
             }
             Api::CType { .. } => RsCodegenResult {
                 global_items: Vec::new(),
@@ -542,38 +544,45 @@ impl<'a> RsCodeGenerator<'a> {
                     type #id = autocxx::#id;
                 })],
                 bindgen_mod_items: Vec::new(),
-                materialization: Use::Unused,
+                materializations: Vec::new(),
                 extern_rust_mod_items: Vec::new(),
             },
-            Api::RustType { .. } => {
-                let id = api.name().get_final_ident();
-                RsCodegenResult {
-                    extern_c_mod_items: Vec::new(),
-                    bridge_items: Vec::new(),
-                    bindgen_mod_items: Vec::new(),
-                    materialization: Use::Unused,
-                    global_items: vec![parse_quote! {
-                        use super::#id;
-                    }],
-                    impl_entry: None,
-                    extern_rust_mod_items: vec![parse_quote! {
-                        type #id;
-                    }],
-                }
-            }
-            Api::RustSubclassFn { details, .. } => Self::generate_subclass_fn(
-                details.params,
-                details.ret,
-                id,
-                details.method_name,
-                details.superclass,
-                details.receiver_mutability,
-            ),
+            Api::RustType { path, .. } => RsCodegenResult {
+                extern_c_mod_items: Vec::new(),
+                bridge_items: Vec::new(),
+                bindgen_mod_items: Vec::new(),
+                materializations: Vec::new(),
+                global_items: vec![parse_quote! {
+                    use super::#path;
+                }],
+                impl_entry: None,
+                extern_rust_mod_items: vec![parse_quote! {
+                    type #id;
+                }],
+            },
+            Api::RustFn { sig, path, .. } => RsCodegenResult {
+                extern_c_mod_items: Vec::new(),
+                bridge_items: Vec::new(),
+                bindgen_mod_items: Vec::new(),
+                materializations: Vec::new(),
+                global_items: vec![parse_quote! {
+                    use super::#path;
+                }],
+                impl_entry: None,
+                extern_rust_mod_items: vec![parse_quote! {
+                    #sig;
+                }],
+            },
+            Api::RustSubclassFn {
+                details, subclass, ..
+            } => Self::generate_subclass_fn(id, *details, subclass),
             Api::Subclass {
                 name, superclass, ..
             } => {
-                let methods = associated_methods.get(&superclass.get_final_item().to_string());
-                self.generate_subclass(name, &superclass, methods)
+                let methods = associated_methods.get(&superclass);
+                let generate_peer_constructor =
+                    subclasses_with_a_single_trivial_constructor.contains(&name.0.name);
+                self.generate_subclass(name, &superclass, methods, generate_peer_constructor)
             }
             Api::RustSubclassConstructor { .. } => RsCodegenResult::default(),
             Api::IgnoredItem { err, ctx, .. } => Self::generate_error_entry(err, ctx),
@@ -585,25 +594,30 @@ impl<'a> RsCodeGenerator<'a> {
         sub: SubclassName,
         superclass: &QualifiedName,
         methods: Option<&Vec<SuperclassMethod>>,
+        generate_peer_constructor: bool,
     ) -> RsCodegenResult {
         let super_name = superclass.get_final_item();
-        let super_id = superclass.get_final_ident();
+        let super_path = superclass.to_type_path();
+        let super_cxxxbridge_id = superclass.get_final_ident();
         let id = sub.id();
         let holder = sub.holder();
-        let cpp = sub.cpp();
-        let full_cpp = QualifiedName::new(&Namespace::new(), cpp.clone());
-        let mut global_items = self.generate_extern_type_impl(TypeKind::NonPod, &full_cpp);
+        let full_cpp = sub.cpp();
+        let cpp_path = full_cpp.to_type_path();
+        let cpp_id = full_cpp.get_final_ident();
+        let mut global_items = Vec::new();
         global_items.push(parse_quote! {
             pub use bindgen::root::#holder;
         });
         let relinquish_ownership_call = sub.cpp_remove_ownership();
         let mut bindgen_mod_items = vec![
-            Item::Struct(new_non_pod_struct(cpp.clone())),
+            parse_quote! {
+                pub use cxxbridge::#cpp_id;
+            },
             parse_quote! {
                 pub struct #holder(pub autocxx::subclass::CppSubclassRustPeerHolder<super::super::super::#id>);
             },
             parse_quote! {
-                impl autocxx::subclass::CppSubclassCppPeer for #cpp {
+                impl autocxx::subclass::CppSubclassCppPeer for #cpp_id {
                     fn relinquish_ownership(&self) {
                         self.#relinquish_ownership_call();
                     }
@@ -611,13 +625,13 @@ impl<'a> RsCodeGenerator<'a> {
             },
         ];
         let mut extern_c_mod_items = vec![
-            ForeignItem::Verbatim(self.generate_cxxbridge_type(&full_cpp)),
+            self.generate_cxxbridge_type(&full_cpp, false, None),
             parse_quote! {
-                fn #relinquish_ownership_call(self: &#cpp);
+                fn #relinquish_ownership_call(self: &#cpp_id);
             },
         ];
         if let Some(methods) = methods {
-            let supers = make_ident(format!("{}_supers", super_name));
+            let supers = SubclassName::get_supers_trait_name(superclass).to_type_path();
             let methods_impls: Vec<ImplItem> = methods
                 .iter()
                 .map(|m| {
@@ -632,8 +646,9 @@ impl<'a> RsCodeGenerator<'a> {
                     let peer_fn = make_ident(peer_fn);
                     *(params.iter_mut().next().unwrap()) = first_param;
                     let param_names = m.param_names.iter().skip(1);
+                    let unsafe_token = get_unsafe_token(m.requires_unsafe);
                     parse_quote! {
-                        fn #supern(#params) #ret {
+                        #unsafe_token fn #supern(#params) #ret {
                             use autocxx::subclass::CppSubclass;
                             self.#peer_fn().#cpp_method_name(#(#param_names),*)
                         }
@@ -647,19 +662,28 @@ impl<'a> RsCodeGenerator<'a> {
                 }
             });
         }
+        if generate_peer_constructor {
+            bindgen_mod_items.push(parse_quote! {
+                impl autocxx::subclass::CppPeerConstructor<#cpp_id> for super::super::super::#id {
+                    fn make_peer(&mut self, peer_holder: autocxx::subclass::CppSubclassRustPeerHolder<Self>) -> cxx::UniquePtr<#cpp_path> {
+                        #cpp_id :: make_unique(peer_holder)
+                    }
+                }
+            })
+        };
 
         // Once for each superclass, in future...
         let as_id = make_ident(format!("As_{}", super_name));
         extern_c_mod_items.push(parse_quote! {
-            fn #as_id(self: &#cpp) -> &#super_id;
+            fn #as_id(self: &#cpp_id) -> &#super_cxxxbridge_id;
         });
         let as_mut_id = make_ident(format!("As_{}_mut", super_name));
         extern_c_mod_items.push(parse_quote! {
-            fn #as_mut_id(self: Pin<&mut #cpp>) -> Pin<&mut #super_id>;
+            fn #as_mut_id(self: Pin<&mut #cpp_id>) -> Pin<&mut #super_cxxxbridge_id>;
         });
         bindgen_mod_items.push(parse_quote! {
-            impl AsRef<#super_id> for super::super::super::#id {
-                fn as_ref(&self) -> &cxxbridge::#super_id {
+            impl AsRef<#super_path> for super::super::super::#id {
+                fn as_ref(&self) -> &cxxbridge::#super_cxxxbridge_id {
                     use autocxx::subclass::CppSubclass;
                     self.peer().#as_id()
                 }
@@ -668,7 +692,7 @@ impl<'a> RsCodeGenerator<'a> {
         // TODO it would be nice to impl AsMut here but pin prevents us
         bindgen_mod_items.push(parse_quote! {
             impl super::super::super::#id {
-                pub fn pin_mut(&mut self) -> std::pin::Pin<&mut cxxbridge::#super_id> {
+                pub fn pin_mut(&mut self) -> std::pin::Pin<&mut cxxbridge::#super_cxxxbridge_id> {
                     use autocxx::subclass::CppSubclass;
                     self.peer_mut().#as_mut_id()
                 }
@@ -683,11 +707,11 @@ impl<'a> RsCodeGenerator<'a> {
         });
         RsCodegenResult {
             extern_c_mod_items,
-            bridge_items: create_impl_items(&cpp, self.config),
+            bridge_items: create_impl_items(&cpp_id, self.config),
             bindgen_mod_items,
-            materialization: Use::Custom(Box::new(parse_quote! {
-                pub use cxxbridge::#cpp;
-            })),
+            materializations: vec![Use::Custom(Box::new(parse_quote! {
+                pub use cxxbridge::#cpp_id;
+            }))],
             global_items,
             impl_entry: None,
             extern_rust_mod_items: vec![
@@ -702,48 +726,54 @@ impl<'a> RsCodeGenerator<'a> {
     }
 
     fn generate_subclass_fn(
-        params: Punctuated<FnArg, Comma>,
-        ret: ReturnType,
         api_name: Ident,
-        method_name: Ident,
-        superclass: QualifiedName,
-        receiver_mutability: ReceiverMutability,
+        details: RustSubclassFnDetails,
+        subclass: SubclassName,
     ) -> RsCodegenResult {
-        let global_def = quote! { fn #api_name(#params) #ret };
+        let params = details.params;
+        let ret = details.ret;
+        let unsafe_token = get_unsafe_token(details.requires_unsafe);
+        let global_def = quote! { #unsafe_token fn #api_name(#params) #ret };
         let params = unqualify_params(params);
         let ret = unqualify_ret_type(ret);
-        let cxxbridge_decl: ForeignItemFn = parse_quote! { fn #api_name(#params) #ret; };
+        let method_name = details.method_name;
+        let cxxbridge_decl: ForeignItemFn =
+            parse_quote! { #unsafe_token fn #api_name(#params) #ret; };
         let args: Punctuated<Expr, Comma> =
             Self::args_from_sig(&cxxbridge_decl.sig.inputs).collect();
-        let superclass_id = superclass.get_final_ident();
-        let superclass_id = make_ident(format!("{}_methods", superclass_id.to_string()));
-        let (deref_ty, deref_call, borrow, mut_token) = match receiver_mutability {
-            ReceiverMutability::Const => ("Deref", "deref", "borrow", None),
+        let superclass_id = details.superclass.get_final_ident();
+        let methods_trait = SubclassName::get_methods_trait_name(&details.superclass);
+        let methods_trait = methods_trait.to_type_path();
+        let (deref_ty, deref_call, borrow, mut_token) = match details.receiver_mutability {
+            ReceiverMutability::Const => ("Deref", "deref", "try_borrow", None),
             ReceiverMutability::Mutable => (
                 "DerefMut",
                 "deref_mut",
-                "borrow_mut",
+                "try_borrow_mut",
                 Some(syn::token::Mut(Span::call_site())),
             ),
         };
         let deref_ty = make_ident(deref_ty);
         let deref_call = make_ident(deref_call);
         let borrow = make_ident(borrow);
+        let destroy_panic_msg = format!("Rust subclass API (method {} of subclass {} of superclass {}) called after subclass destroyed", method_name, subclass.0.name, superclass_id);
+        let reentrancy_panic_msg = format!("Rust subclass API (method {} of subclass {} of superclass {}) called whilst subclass already borrowed - likely a re-entrant call",  method_name, subclass.0.name, superclass_id);
         RsCodegenResult {
             extern_c_mod_items: Vec::new(),
             bridge_items: Vec::new(),
             bindgen_mod_items: Vec::new(),
-            materialization: Use::Unused,
+            materializations: Vec::new(),
             global_items: vec![parse_quote! {
                 #global_def {
                     let rc = me.0
                         .get()
-                        .expect("Rust subclass destroyed prior to a method being called");
+                        .expect(#destroy_panic_msg);
                     let #mut_token b = rc
                         .as_ref()
-                        .#borrow();
+                        .#borrow()
+                        .expect(#reentrancy_panic_msg);
                     let r = std::ops::#deref_ty::#deref_call(& #mut_token b);
-                    #superclass_id :: #method_name
+                    #methods_trait :: #method_name
                         (r,
                         #args)
                 }
@@ -767,20 +797,77 @@ impl<'a> RsCodeGenerator<'a> {
             .flatten()
     }
 
-    fn generate_type<T, F>(
+    fn generate_type<F>(
         &self,
         name: &QualifiedName,
         id: Ident,
-        item: T,
-        analysis: TypeKind,
-        item_type: F,
-        methods: Option<&Vec<SuperclassMethod>>,
+        type_kind: TypeKind,
+        item_creator: F,
+        associated_methods: &HashMap<QualifiedName, Vec<SuperclassMethod>>,
     ) -> RsCodegenResult
     where
-        F: FnOnce(T) -> Item,
+        F: FnOnce() -> Option<(Item, Option<Attribute>)>,
     {
-        let mut bindgen_mod_items = vec![item_type(item)];
-        let mut global_items = self.generate_extern_type_impl(analysis, name);
+        let mut bindgen_mod_items = Vec::new();
+        let mut materializations = vec![Use::UsedFromCxxBridge];
+        Self::add_superclass_stuff_to_type(
+            name,
+            &mut bindgen_mod_items,
+            &mut materializations,
+            associated_methods.get(name),
+        );
+        let orig_item = item_creator();
+        match type_kind {
+            TypeKind::Pod | TypeKind::NonPodNested => {
+                let mut item = orig_item
+                    .expect("Instantiable types must provide instance")
+                    .0;
+                if matches!(type_kind, TypeKind::NonPodNested) {
+                    // We have to use 'type A = super::bindgen::A::B'
+                    // because if we use simply 'type A', there is no combination
+                    // of cxx-acceptable attributes which will inform cxx that
+                    // A is a class rather than a namespace.
+                    if let Item::Struct(ref mut s) = item {
+                        // Retain generics and doc attrs.
+                        make_non_pod(s);
+                    } else {
+                        // enum
+                        item = Item::Struct(new_non_pod_struct(id.clone()));
+                    }
+                }
+                bindgen_mod_items.push(item);
+                RsCodegenResult {
+                    global_items: self.generate_extern_type_impl(type_kind, name),
+                    impl_entry: None,
+                    bridge_items: create_impl_items(&id, self.config),
+                    extern_c_mod_items: vec![self.generate_cxxbridge_type(name, true, None)],
+                    bindgen_mod_items,
+                    materializations,
+                    extern_rust_mod_items: Vec::new(),
+                }
+            }
+            TypeKind::NonPod | TypeKind::Abstract => {
+                bindgen_mod_items.push(Item::Use(parse_quote! { pub use cxxbridge::#id; }));
+                let doc_attr = orig_item.map(|maybe_item| maybe_item.1).flatten();
+                RsCodegenResult {
+                    extern_c_mod_items: vec![self.generate_cxxbridge_type(name, false, doc_attr)],
+                    extern_rust_mod_items: Vec::new(),
+                    bridge_items: Vec::new(),
+                    global_items: Vec::new(),
+                    bindgen_mod_items,
+                    impl_entry: None,
+                    materializations,
+                }
+            }
+        }
+    }
+
+    fn add_superclass_stuff_to_type(
+        name: &QualifiedName,
+        bindgen_mod_items: &mut Vec<Item>,
+        materializations: &mut Vec<Use>,
+        methods: Option<&Vec<SuperclassMethod>>,
+    ) {
         if let Some(methods) = methods {
             let (supers, mains): (Vec<_>, Vec<_>) = methods
                 .iter()
@@ -795,19 +882,20 @@ impl<'a> RsCodeGenerator<'a> {
                         ReceiverMutability::Mutable => parse_quote!(&mut self),
                     };
                     let ret_type = &method.ret_type;
+                    let unsafe_token = get_unsafe_token(method.requires_unsafe);
                     let a: TraitItem = parse_quote!(
-                        fn #super_id(#params) #ret_type;
+                        #unsafe_token fn #super_id(#params) #ret_type;
                     );
                     let b: TraitItem = parse_quote!(
-                        fn #id(#params) #ret_type {
+                        #unsafe_token fn #id(#params) #ret_type {
                             self.#super_id(#param_names)
                         }
                     );
                     (a, b)
                 })
                 .unzip();
-            let supers_name = make_ident(format!("{}_supers", id.to_string()));
-            let methods_name = make_ident(format!("{}_methods", id.to_string()));
+            let supers_name = SubclassName::get_supers_trait_name(name).get_final_ident();
+            let methods_name = SubclassName::get_methods_trait_name(name).get_final_ident();
             bindgen_mod_items.push(parse_quote! {
                 #[allow(non_snake_case)]
                 pub trait #supers_name {
@@ -820,22 +908,10 @@ impl<'a> RsCodeGenerator<'a> {
                     #(#mains)*
                 }
             });
-            global_items.push(parse_quote! {
-                pub use bindgen::root::#methods_name;
-            });
-        }
-        RsCodegenResult {
-            global_items,
-            impl_entry: None,
-            bridge_items: if analysis.can_be_instantiated() {
-                create_impl_items(&id, self.config)
-            } else {
-                Vec::new()
-            },
-            extern_c_mod_items: vec![ForeignItem::Verbatim(self.generate_cxxbridge_type(name))],
-            bindgen_mod_items,
-            materialization: Use::UsedFromCxxBridge,
-            extern_rust_mod_items: Vec::new(),
+            materializations.extend([
+                Use::SpecificNameFromBindgen(methods_name),
+                Use::SpecificNameFromBindgen(supers_name),
+            ]);
         }
     }
 
@@ -849,10 +925,10 @@ impl<'a> RsCodeGenerator<'a> {
                 let id = Self::sanitize_error_ident(&id).unwrap_or(id);
                 (
                     None,
-                    Use::Custom(Box::new(parse_quote! {
+                    Some(Use::Custom(Box::new(parse_quote! {
                         #[doc = #err]
                         pub struct #id;
-                    })),
+                    }))),
                 )
             }
             ErrorContext::Method { self_ty, method }
@@ -871,7 +947,7 @@ impl<'a> RsCodeGenerator<'a> {
                         },
                         ty: self_ty,
                     })),
-                    Use::Unused,
+                    None,
                 )
             }
             ErrorContext::Method { self_ty, method } => {
@@ -883,10 +959,10 @@ impl<'a> RsCodeGenerator<'a> {
                 ));
                 (
                     None,
-                    Use::Custom(Box::new(parse_quote! {
+                    Some(Use::Custom(Box::new(parse_quote! {
                         #[doc = #err]
                         pub struct #id;
-                    })),
+                    }))),
                 )
             }
         };
@@ -896,7 +972,7 @@ impl<'a> RsCodeGenerator<'a> {
             bridge_items: Vec::new(),
             extern_c_mod_items: Vec::new(),
             bindgen_mod_items: Vec::new(),
-            materialization,
+            materializations: materialization.into_iter().collect(),
             extern_rust_mod_items: Vec::new(),
         }
     }
@@ -950,9 +1026,18 @@ impl<'a> RsCodeGenerator<'a> {
         })]
     }
 
-    fn generate_cxxbridge_type(&self, name: &QualifiedName) -> TokenStream {
+    fn generate_cxxbridge_type(
+        &self,
+        name: &QualifiedName,
+        references_bindgen: bool,
+        doc_attr: Option<Attribute>,
+    ) -> ForeignItem {
         let ns = name.get_namespace();
         let id = name.get_final_ident();
+        // The following lines actually Tell A Lie.
+        // If we have a nested class, B::C, within namespace A,
+        // we actually have to tell cxx that we have nested class C
+        // within namespace A.
         let mut ns_components: Vec<_> = ns.iter().cloned().collect();
         let mut cxx_name = None;
         if let Some(cpp_name) = self.original_name_map.get(name) {
@@ -976,18 +1061,28 @@ impl<'a> RsCodeGenerator<'a> {
             });
         }
 
-        for_extern_c_ts.extend(quote! {
-            type #id = super::bindgen::root::
-        });
-        for_extern_c_ts.extend(ns.iter().map(make_ident).map(|id| {
-            quote! {
-                #id::
-            }
-        }));
-        for_extern_c_ts.extend(quote! {
-            #id;
-        });
-        for_extern_c_ts
+        if let Some(doc_attr) = doc_attr {
+            doc_attr.to_tokens(&mut for_extern_c_ts);
+        }
+
+        if references_bindgen {
+            for_extern_c_ts.extend(quote! {
+                type #id = super::bindgen::root::
+            });
+            for_extern_c_ts.extend(ns.iter().map(make_ident).map(|id| {
+                quote! {
+                    #id::
+                }
+            }));
+            for_extern_c_ts.extend(quote! {
+                #id;
+            });
+        } else {
+            for_extern_c_ts.extend(quote! {
+                type #id;
+            });
+        }
+        ForeignItem::Verbatim(for_extern_c_ts)
     }
 
     fn find_output_mod_root(ns: &Namespace) -> impl Iterator<Item = Ident> {
@@ -995,6 +1090,37 @@ impl<'a> RsCodeGenerator<'a> {
     }
 }
 
+fn get_unsafe_token(requires_unsafe: bool) -> TokenStream {
+    if requires_unsafe {
+        quote! { unsafe }
+    } else {
+        quote! {}
+    }
+}
+
+fn find_trivially_constructed_subclasses(apis: &[Api<FnPhase>]) -> HashSet<QualifiedName> {
+    let (simple_constructors, complex_constructors): (Vec<_>, Vec<_>) = apis
+        .iter()
+        .map(|api| match api {
+            Api::RustSubclassConstructor {
+                subclass,
+                is_trivial,
+                ..
+            } => Some((&subclass.0.name, is_trivial)),
+            _ => None,
+        })
+        .flatten()
+        .partition(|(_, trivial)| **trivial);
+    let simple_constructors: HashSet<_> =
+        simple_constructors.into_iter().map(|(qn, _)| qn).collect();
+    let complex_constructors: HashSet<_> =
+        complex_constructors.into_iter().map(|(qn, _)| qn).collect();
+    (&simple_constructors - &complex_constructors)
+        .into_iter()
+        .cloned()
+        .collect()
+}
+
 impl HasNs for (QualifiedName, RsCodegenResult) {
     fn get_namespace(&self) -> &Namespace {
         self.0.get_namespace()
@@ -1017,5 +1143,5 @@ struct RsCodegenResult {
     global_items: Vec<Item>,
     bindgen_mod_items: Vec<Item>,
     impl_entry: Option<Box<ImplBlockDetails>>,
-    materialization: Use,
+    materializations: Vec<Use>,
 }
