@@ -44,7 +44,7 @@ pub enum ConvertError {
     Blocked(QualifiedName),
     UnusedTemplateParam,
     TooManyUnderscores,
-    UnknownDependentType,
+    UnknownDependentType(QualifiedName),
     IgnoredDependent,
     MoveConstructorUnsupported,
     ReservedName,
@@ -52,7 +52,7 @@ pub enum ConvertError {
     UnsupportedReceiver,
     BoxContainingNonRustType(QualifiedName),
     RustTypeWithAPath(QualifiedName),
-    SuperclassConstructorsTooFancy(QualifiedName),
+    AbstractNestedType,
 }
 
 fn format_maybe_identifier(id: &Option<Ident>) -> String {
@@ -89,7 +89,7 @@ impl Display for ConvertError {
             ConvertError::Blocked(tn) => write!(f, "Found an attempt at using a type marked as blocked! ({})", tn.to_cpp_name())?,
             ConvertError::UnusedTemplateParam => write!(f, "This function or method uses a type where one of the template parameters was incomprehensible to bindgen/autocxx - probably because it uses template specialization.")?,
             ConvertError::TooManyUnderscores => write!(f, "Names containing __ are reserved by C++ so not acceptable to cxx")?,
-            ConvertError::UnknownDependentType => write!(f, "This item relies on a type not known to autocxx.")?,
+            ConvertError::UnknownDependentType(qn) => write!(f, "This item relies on a type not known to autocxx ({})", qn.to_cpp_name())?,
             ConvertError::IgnoredDependent => write!(f, "This item depends on some other type which autocxx could not generate.")?,
             ConvertError::MoveConstructorUnsupported => write!(f, "This is a move constructor, for which we currently cannot generate bindings.")?,
             ConvertError::ReservedName => write!(f, "The item name is a reserved word in Rust.")?,
@@ -97,7 +97,7 @@ impl Display for ConvertError {
             ConvertError::UnsupportedReceiver => write!(f, "This is a method on a type which can't be used as the receiver in Rust (i.e. self/this). This is probably because some type involves template specialization.")?,
             ConvertError::BoxContainingNonRustType(ty) => write!(f, "A rust::Box<T> was encountered where T was not known to be a Rust type. Use rust_type!(T): {}", ty.to_cpp_name())?,
             ConvertError::RustTypeWithAPath(ty) => write!(f, "A qualified Rust type was found (i.e. one containing ::): {}. Rust types must always be a simple identifier.", ty.to_cpp_name())?,
-            ConvertError::SuperclassConstructorsTooFancy(ty) => write!(f, "It's not yet supported to create subclasses of superclasses with anything other than a single, zero-argument constructor. {} doesn't meet this rule. This is a temporary limitation - see https://github.com/google/autocxx/issues/596.", ty.to_cpp_name())?,
+            ConvertError::AbstractNestedType => write!(f, "This type is nested within another struct/class, yet is abstract (or is not on the allowlist so we can't be sure). This is not yet supported by autocxx. If you don't believe this type is abstract, add it to the allowlist.")?,
         }
         Ok(())
     }
